/***************************************************************************
 *   Copyright (C) 2004 by Gerd Fleischer                                  *
 *   gerdfleischer@web.de                                                  *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "klinpopup.h"
#include "settings.h"
#include "prefs.h"
#include "makepopup.h"

#include <qdragobject.h>
#include <qpainter.h>
#include <qpaintdevicemetrics.h>
#include <qtooltip.h>
#include <qdir.h>

#include <kwin.h>
#include <kdebug.h>
#include <kconfigdialog.h>
#include <kglobal.h>
#include <klocale.h>
#include <kiconloader.h>
#include <kdeversion.h>
#include <kmenubar.h>
#include <kstatusbar.h>
#include <kkeydialog.h>
#include <kaudioplayer.h>
#include <kmessagebox.h>
#include <kedittoolbar.h>
#include <kprocess.h>
#include <kfileitem.h>

#include <kstdaccel.h>
#include <kstdaction.h>

KLinPopup::KLinPopup()
    : KMainWindow( 0, "KLinPopup" ),
      m_view(new KLinPopupView(this)),
      actualPopup(0),
      unreadMessages(0),
      popupFileDirectory("/var/lib/klinpopup")    
{
    setFocusPolicy(QWidget::StrongFocus);
    
    // accept dnd
    setAcceptDrops(true);

    // tell the KMainWindow that this is indeed the main widget
    setCentralWidget(m_view);

    // then, setup our actions
    setupActions();

    // and a status bar
    statusBar()->show();
    
    // apply the saved mainwindow settings, if any, and ask the mainwindow
    // to automatically save settings if changed: window size, toolbar
    // position, icon size, etc.
    setAutoSaveSettings();

    readConfig();
    
    initSystemTray();

    // allow the view to change the statusbar and caption
    connect(m_view, SIGNAL(signalChangeStatusbar(const QString&)),
            this,   SLOT(changeStatusbar(const QString&)));
    connect(m_view, SIGNAL(signalChangeCaption(const QString&)),
            this,   SLOT(changeCaption(const QString&)));
   
    updateStats();

//    statusBar()->hide();
//    m_statusbarAction->setChecked(false);
    
    initTimer();
    popupFileTimer->start(1, true);
}

KLinPopup::~KLinPopup()
{
}

void KLinPopup::closeEvent(QCloseEvent *e)
{
    // copied from kscd
    const QObject* caller = sender();
    
    while (m_systemTray && caller)
    {
        if (caller == m_systemTray)
        {
            break;
        }
        caller = caller->parent();
    }

    if (!caller)
    {
        hide();
        e->ignore();
        return;
    }

    e->accept();
    if (e->isAccepted())
        kapp->quit();
}

void KLinPopup::hideEvent(QHideEvent *e)
{
    KMainWindow::hideEvent(e);
    m_systemTray->show();
    hide();
}

void KLinPopup::focusInEvent(QFocusEvent *e)
{
    kdDebug() << "got focus" << endl;
    if (e->gotFocus()) {
        if (!checkReadStatus()) {
            unreadMessages--;
            setRead();
        }    
        updateStats();
        if (unreadMessages == 0)
            m_systemTray->changeTrayPixmap(NORMAL_ICON);
    }
}

void KLinPopup::setupActions()
{
    KStdAction::quit(kapp, SLOT(quit()), actionCollection());

//    KStdAction::copy(kapp, SLOT(copy()), actionCollection());
    
    m_toolbarAction = KStdAction::showToolbar(this, SLOT(optionsShowToolbar()), actionCollection());
    m_statusbarAction = KStdAction::showStatusbar(this, SLOT(optionsShowStatusbar()), actionCollection());

    KStdAction::keyBindings(this, SLOT(optionsConfigureKeys()), actionCollection());
    KStdAction::configureToolbars(this, SLOT(optionsConfigureToolbars()), actionCollection());
    KStdAction::preferences(this, SLOT(optionsPreferences()), actionCollection());

    newPopupAction = new KAction(i18n("&New"),
                                      "mail_new", 0,
                                      this, SLOT(newPopup()),
                                      actionCollection(), "new_popup");
    replyPopupAction = new KAction(i18n("&Reply"),
                                        "mail_reply", 0,
                                        this, SLOT(replyPopup()),
                                        actionCollection(), "reply_popup");
    previousPopupAction = new KAction(i18n("&Previous"),
                                           "previous", 0,
                                           this, SLOT(previousPopup()),
                                           actionCollection(), "previous_popup");
    nextPopupAction = new KAction(i18n("&Next"),
                                       "next", 0,
                                       this, SLOT(nextPopup()),
                                       actionCollection(), "next_popup");
    deletePopupAction = new KAction(i18n("&Delete"),
                                         "mail_delete", 0,
                                         this, SLOT(deletePopup()),
                                         actionCollection(), "delete_popup");
    
    createGUI();
}

void KLinPopup::initSystemTray()
{
    m_systemTray = new SystemTray(this, "systemTray");
    m_systemTray->show();
}

void KLinPopup::initTimer()
{
    // Popupfile Timer
    popupFileTimer = new QTimer(this);
    connect(popupFileTimer, SIGNAL(timeout()), this, SLOT(popupFileTimerDone()));
}

//should return true in almost every situation
//Simplify this if/else mess??
bool KLinPopup::checkPopupFileDirectory()
{
    QDir dir(popupFileDirectory);
    if (! dir.exists()) {
        int tmpYesNo =  KMessageBox::warningYesNo(this, i18n("Working directory /var/lib/klinpopup/ does not exist!\n"
                                                             "Shall I create it? (May need root password)"));
        if (tmpYesNo == KMessageBox::Yes) {
            QString kdesuCmd = "mkdir -p -m 0777 /var/lib/klinpopup";
            KProcess *p;
            p = new KProcess();
            *p << "kdesu" << "-c" << kdesuCmd;
            if (p->start(KProcess::Block)) {
///@TODO: is this the return value of kdesu or what?
                return true;
            } else {
                return false;
            }
        } else {
            return false;
        }        
    } else {
        KFileItem tmpFileItem = KFileItem(KFileItem::Unknown, KFileItem::Unknown, "/var/lib/klinpopup"); 
        QString tmpPerms = tmpFileItem.permissionsString();
//        kdDebug() << tmpPerms.latin1() << endl;
        if (tmpPerms != "drwxrwxrwx") {
            kdDebug() << "Perms not ok!" << endl;
            int tmpYesNo =  KMessageBox::warningYesNo(this, i18n("Permissions of the working directory /var/lib/klinpopup/ are wrong!\n"
                                                                 "Fix? (May need root password)"));
            if (tmpYesNo == KMessageBox::Yes) {
                QString kdesuCmd = "chmod 0777 /var/lib/klinpopup";
                KProcess *p;
                p = new KProcess();
                *p << "kdesu" << "-c" << kdesuCmd;
                if (p->start(KProcess::Block)) {
                    return true;
                } else {
                    return false;
                }        
            } else {
                return false;
            }        
        } else {
            return true;
        }            
    }         
}

void KLinPopup::popupFileTimerDone()
{
    if (checkPopupFileDirectory()) {
        QDir dir(popupFileDirectory);
        const QFileInfoList *popupFiles = dir.entryInfoList(QDir::All, QDir::Time);
        if (popupFiles) {
            QFileInfoListIterator it(*popupFiles);
            QFileInfo *popupFileInfo;
            while((popupFileInfo = it.current()) != 0) {
                ++it;
                if (popupFileInfo -> isFile()) {
                    QString popupFileName(popupFileInfo -> fileName());
                    QString popupFilePath(popupFileDirectory);
                    popupFilePath.append("/");
                    popupFilePath.append(popupFileName);
    
                    QFile popupFile(popupFilePath);

                    kdDebug() << "popupFile " << popupFileName.latin1() << endl;

                    if (popupFile.open(IO_ReadOnly)) {
                        QTextStream stream(&popupFile);
                        QString line;
                        QString time;
                        QString sender;
                        QString machine;
                        QString ip;
                        QString text;
                        int i = 0;

                        while (! stream.atEnd()) {
                            i++;
                            line = stream.readLine();
    
                            if (i == 1)  {
                                time.append(line);
                                kdDebug() << "Time " << time.latin1() << endl;
                                continue;
                            }
                            if (i == 2) {
                                sender.append(line);
                                kdDebug() << "Sender " << sender.latin1() << endl;
                                continue;
                            }
                            if (i == 3) {
                                machine.append(line);
                                kdDebug() << "Machine " << machine.latin1() << endl;
                                continue;
                            }
                            if (i == 4) {
                                ip.append(line);
                                kdDebug() << "IP " << ip.latin1() << endl;
                                continue;
                            }
                            if (i > 5) {
                                text.append('\n');
                            }
                            text.append(line);
                        }
                            
                        popupFile.close();
                        kdDebug() << "Closed" << endl;

                        // delete file
                        if (! popupFile.remove()) {
                            ///@TODO: handle error
                            perror("can't remove message");
                            kdDebug() << "Deleted" << endl;
                        }

                        signalNewMessage(sender, time, machine, ip, text);
                    }
                }
            }
        }
        popupFileTimer->start(timerIntervalSecs * 1000, true);
    } else {
        int tmpContinueQuit = KMessageBox::warningYesNo(this,
                                                        i18n("There is a serious problem with the working directory!\n"
                                                        "Only sending messages will work, "
                                                        "else you can manually fix and restart KLinPopup."),
                                                        i18n("Warning"),
                                                        i18n("&Continue"),
                                                        i18n("&Quit"),
                                                        "ShowWarningContinueQuit");
        if (tmpContinueQuit != KMessageBox::Yes) {
            kdDebug() << "quit" << endl;;
            kapp->exit(1);
        }
    }
}    

void KLinPopup::signalNewMessage(QString popupSender, QString popupTime, QString popupMachine, QString popupIp, QString messageText)
{
    kdDebug() << "Popup received" << endl;

    int tmpCount;    
    tmpCount = popupSenderMap.count() + 1;
    popupSenderMap.insert(tmpCount, popupSender);
    popupTimeMap.insert(tmpCount, popupTime);
    popupMachineMap.insert(tmpCount, popupMachine);
    popupIpMap.insert(tmpCount, popupIp);
    popupMessageTextMap.insert(tmpCount, messageText);
    readStatus.insert(tmpCount, 1);
    unreadMessages++;
    
    switch (newMessageSignaling)
    {
        case MS_NOTHING:    //nothing
            //show it if it's the only one
            if (popupSenderMap.count() == 1) {
                actualPopup = 1;
                showPopup();
            }
            updateStats();
            break;
        case MS_SOUND_TRAY:    //play sound
            actualPopup = popupSenderMap.count();
            showPopup();    
            KAudioPlayer::play(newPopupSound);
            if (isActiveWindow()) {
                unreadMessages--;
                setRead();
                if (unreadMessages == 0)
                    m_systemTray->changeTrayPixmap(NORMAL_ICON);
            } else {
                m_systemTray->changeTrayPixmap(NEW_ICON);
            }
            updateStats();
            break;
        case MS_ACTIVATE:    //activate Window
            actualPopup = popupSenderMap.count();
            showPopup();
            show();
            KWin::forceActiveWindow(winId());
            unreadMessages--;
            setRead();
            if (unreadMessages == 0)
                m_systemTray->changeTrayPixmap(NORMAL_ICON);
            updateStats();
            break;
        case MS_ALL:    //all
            actualPopup = popupSenderMap.count();
            showPopup();
            KAudioPlayer::play(newPopupSound);
            show();
            KWin::forceActiveWindow(winId());
            unreadMessages--;
            setRead();
            if (unreadMessages == 0)
                m_systemTray->changeTrayPixmap(NORMAL_ICON);
            updateStats();
            break;
    }      
}

void KLinPopup::createStatText()
{
    QString tmpNumber;
    
    captionText = "";
    tmpNumber.setNum(actualPopup, 10);
    captionText.append(tmpNumber);
    captionText.append("/");
    tmpNumber.setNum(popupSenderMap.count(), 10);
    captionText.append(tmpNumber);
    
    statText.setNum(unreadMessages, 10);
    statText.prepend(i18n("Message ") + captionText + " - ");
    statText.append(i18n(" Unread"));
    
}

void KLinPopup::updateStats()
{
    createStatText();
    changeCaption(captionText);
    changeStatusbar(statText);
    checkMessageMap();
    QToolTip::remove(m_systemTray);
    QToolTip::add(m_systemTray, statText);
}

QString KLinPopup::createSenderText()
{
    QString tmpSenderText = "";
    if (displaySender == true) {
        tmpSenderText.append(popupSenderMap[actualPopup]);
    }
    if (displayMachine == true) {
        if (!tmpSenderText.isEmpty()) {
            tmpSenderText.append("/");
        }
        tmpSenderText.append(popupMachineMap[actualPopup]);
    }
    if (displayIP == true) {
        if (!tmpSenderText.isEmpty()) {
            tmpSenderText.append("/");
        }
        tmpSenderText.append(popupIpMap[actualPopup]);
    }
    return tmpSenderText;
}
    
void KLinPopup::checkMessageMap() {
    int popupCounter = popupSenderMap.count();

    if (popupCounter != 0) {
        replyPopupAction->setEnabled(true);
        deletePopupAction->setEnabled(true);
    } else {
        replyPopupAction->setEnabled(false);
        deletePopupAction->setEnabled(false);
    }

    if (popupCounter > 1 && actualPopup > 1) {
        previousPopupAction->setEnabled(true);
    } else {
        previousPopupAction->setEnabled(false);
    }

    if (popupCounter > 1 && actualPopup < popupCounter) {
        nextPopupAction->setEnabled(true);
    } else {
        nextPopupAction->setEnabled(false);
    }
}
    
void KLinPopup::showPopup()
{
    if (!popupSenderMap.isEmpty()) {
        m_view->displayNewMessage(createSenderText(),
                                  popupTimeMap[actualPopup],
                                  popupMessageTextMap[actualPopup]);
    } else {
        m_view->displayNewMessage("", "", "");
    }
}
    
void KLinPopup::newPopup()
{
    makePopup *newPopupView = new makePopup(this, "New Popup", "");
    newPopupView->setCaption(i18n("New Popup"));
    newPopupView->exec();
}

void KLinPopup::replyPopup()
{
    QString sender = popupMachineMap[actualPopup];

    makePopup *replyPopupView = new makePopup(this, "Reply Popup", sender);
    replyPopupView->setCaption(i18n("Reply Popup"));
    replyPopupView->exec();
}

void KLinPopup::previousPopup()
{
    actualPopup--;
    showPopup();
    if (!checkReadStatus()) {
        unreadMessages--;
        setRead();
    }
    if (unreadMessages == 0)
       m_systemTray->changeTrayPixmap(NORMAL_ICON);
       
    updateStats();
}

void KLinPopup::nextPopup()
{
    actualPopup++;
    showPopup();
    if (!checkReadStatus()) {
        unreadMessages--;
        setRead();
    }
    if (unreadMessages == 0)
       m_systemTray->changeTrayPixmap(NORMAL_ICON);
    
    updateStats();   
}

void KLinPopup::deletePopup() {
    int countPopup = popupSenderMap.count();

    if (actualPopup == countPopup) {
        popupSenderMap.remove(actualPopup);
        popupTimeMap.remove(actualPopup);
        popupMachineMap.remove(actualPopup);
        popupIpMap.remove(actualPopup);
        popupMessageTextMap.remove(actualPopup);
        readStatus.remove(actualPopup);
        actualPopup--;
    } else {
        for (int i = actualPopup; i <= countPopup - 1; i++) {
            popupSenderMap.insert(i, popupSenderMap[i + 1]);
            popupSenderMap.remove(i + 1);
            
            popupTimeMap.insert(i, popupTimeMap[i + 1]);
            popupTimeMap.remove(i + 1);
            
            popupMachineMap.insert(i, popupMachineMap[i + 1]);
            popupMachineMap.remove(i + 1);
            
            popupIpMap.insert(i, popupIpMap[i + 1]);
            popupIpMap.remove(i + 1);
            
            popupMessageTextMap.insert(i, popupMessageTextMap[i + 1]);
            popupMessageTextMap.remove(i + 1);
    
            readStatus.insert(i, readStatus[i + 1]);
            readStatus.remove(i + 1);
        }
    }

    showPopup();
    if (!checkReadStatus()) {
        unreadMessages--;
        setRead();
    }
    if (unreadMessages == 0)
        m_systemTray->changeTrayPixmap(NORMAL_ICON);

    updateStats();
}

bool KLinPopup::checkReadStatus()
{
    if (readStatus[actualPopup] == 0) {
        return true;
    } else {
        return false;
    }
}

void KLinPopup::setRead()
{
    readStatus[actualPopup] = 0;
}    

void KLinPopup::optionsShowToolbar()
{
    // this is all very cut and paste code for showing/hiding the
    // toolbar
    if (m_toolbarAction->isChecked())
        toolBar()->show();
    else
        toolBar()->hide();
}

void KLinPopup::optionsShowStatusbar()
{
    // show/hide the statusbar
    if (m_statusbarAction->isChecked())
        statusBar()->show();
    else
        statusBar()->hide();
}

void KLinPopup::optionsConfigureKeys()
{
    KKeyDialog::configure(actionCollection());
}

void KLinPopup::optionsConfigureToolbars()
{
    // use the standard toolbar editor
    saveMainWindowSettings(KGlobal::config(), autoSaveGroup());
}

void KLinPopup::newToolbarConfig()
{
    // this slot is called when user clicks "Ok" or "Apply" in the toolbar editor.
    // recreate our GUI, and re-apply the settings (e.g. "text under icons", etc.)
    createGUI();

    applyMainWindowSettings(KGlobal::config(), autoSaveGroup());
}

void KLinPopup::optionsPreferences()
{
        // The preference dialog is derived from prefs-base.ui which is subclassed into Prefs
        //
        // compare the names of the widgets in the .ui file 
        // to the names of the variables in the .kcfg file
        KConfigDialog *dialog = new KConfigDialog(this, "settings", Settings::self(), KDialogBase::Swallow);
        dialog->addPage(new Prefs(), i18n("General"), "package_settings");
        connect(dialog, SIGNAL(settingsChanged()), this, SLOT(settingsChanged()));
        dialog->show();
}

void KLinPopup::readConfig()
{
    startDocked = Settings::startDocked();
    displaySender = Settings::displaySender();
    displayMachine = Settings::displayMachine();
    displayIP = Settings::displayIP();
    newMessageSignaling = Settings::toggleSignaling();
    newPopupSound = Settings::soundURL();
    timerIntervalSecs= Settings::timerInterval();
}

void KLinPopup::saveConfig()
{
    Settings::writeConfig();
}
  
void KLinPopup::settingsChanged()
{
    Settings::writeConfig();
    displaySender = Settings::displaySender();
    displayMachine = Settings::displayMachine();
    displayIP = Settings::displayIP();
    //Avoid empty senderBox
    if (!displaySender && !displayMachine && !displayIP) {
        Settings::setDisplaySender(true);
        Settings::writeConfig();
        displaySender = Settings::displaySender();
    }    
    newMessageSignaling = Settings::toggleSignaling();
    if (newMessageSignaling == MS_SOUND_TRAY && unreadMessages != 0) {
        m_systemTray->changeTrayPixmap(NEW_ICON);
    }    
    newPopupSound = Settings::soundURL();
    timerIntervalSecs = Settings::timerInterval();
    popupFileTimer->changeInterval(timerIntervalSecs * 1000);
    showPopup();
}

void KLinPopup::changeStatusbar(const QString& text)
{
    if (!statusBar()->hasItem(ID_STATUS_TEXT)) {
        statusBar()->insertItem(text, ID_STATUS_TEXT, 1, true);
    } else {
        statusBar()->changeItem(text, ID_STATUS_TEXT);
    }
}

void KLinPopup::changeCaption(const QString& text)
{
    setCaption(text);
}

#include "klinpopup.moc"

