/***************************************************************************
 *   Copyright (C) 2004 by Gerd Fleischer                                  *
 *   gerdfleischer@web.de                                                  *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include <malloc.h>
#include <stdio.h>
#include <unistd.h>

#include <makepopup.h>

#include <kaction.h>
#include <kdebug.h>
#include <klocale.h>
#include <kmessagebox.h>
#include <kpushbutton.h>
#include <kuser.h>

#include <qfile.h>
#include <qlabel.h>

makePopup::makePopup(QWidget *parent, const char *name, QString sender) : makePopup_view(parent, name, true)
{
    //initialize senderBox, groupBox and hostBox
    senderBox->insertItem(getHostName());
    senderBox->insertItem(KUser().loginName());
    senderBox->insertItem(KUser().fullName());
	
    if (sender.isEmpty()) {
        groupBox->insertItem(i18n("NO GROUP"), -1);
        readGroupList();
    } else {
        groupBox->setEnabled(false);
        hostBox->insertItem(sender);
        hostBox->setEnabled(false);
    }        
    
    connect(groupBox, SIGNAL(activated(const QString &)), SLOT(changeGroupBox(const QString &)));
    connect(buttonSend, SIGNAL(clicked()), SLOT(sendPopup()));
}


makePopup::~makePopup()
{
}


void makePopup::sendPopup()
{
    QString sendCmd;
    sendCmd.append("echo '");
    sendCmd.append(messageText->text());
    sendCmd.append("'|");
    sendCmd.append(SMBPATH);
    sendCmd.append("/smbclient -N ");
    sendCmd.append("-U '");
    sendCmd.append(senderBox->currentText());
    sendCmd.append("' -M '");
    sendCmd.append(hostBox->currentText());
    sendCmd.append("' > /dev/null\n");

    kdDebug() << sendCmd.utf8() << endl;

    // open process
    //UTF-8 or latin1??
    //use KProcess??
    FILE *sendprocess = popen(sendCmd.utf8(), "w");

    if (pclose(sendprocess) == 0) {
        KMessageBox::information(this, i18n("Popup sent!"), i18n("Success"), "ShowPopupSentSuccess");
    } else {
        KMessageBox::sorry(this, i18n("Popup not sent!"), i18n("Error"));
    }
}

/*
 * readHostName
 */
QString makePopup::getHostName()
{
    // todo read name from samba
    char *tmp = (char *) malloc (255 * (sizeof(char)));
    gethostname(tmp, 255);
    QString hostname = tmp;
    hostname = hostname.upper();
    if (hostname.contains('.') != 0) {
        hostname.remove(hostname.find('.'), hostname.length());
    }
    return hostname;
}


/*
 * read available groups from samba (localhost)
 */
void makePopup::readGroupList()
{
    QString cmd;
    cmd.append(SMBPATH);
    cmd.append("/smbclient -L ");
    cmd.append("localhost");
    cmd.append(" -N");

    // open process
    FILE *sambaQuery = popen(cmd.latin1(), "r");

    // read data
    if (sambaQuery != NULL) {
        QTextStream stream (sambaQuery, IO_ReadOnly);
        QString line;
        bool groupLine = false;
        bool sepLine = false;
        while (! stream.eof()) {
            line = stream.readLine();
            if (! groupLine && line.contains("\tWorkgroup            Master", true) == 1) {
                groupLine = true;
            } else if (groupLine && ! sepLine && line.contains("\t---------            -------", true) == 1) {
                sepLine = true;
            } else if (sepLine) {
                if (line.length() == 0) {
                    break;
                }
                line.remove(0, 1);
                QString group = line.mid(0, line.find("  ", 0, true));
                kdDebug() << "found group = '" << group << "'" << endl;
                groupBox->insertItem(group, -1);
            }
        }
    }
    else {
        QString errorMsg;
        errorMsg.append("can't open process='read group list' ");
        errorMsg.append(cmd);
        perror(errorMsg);
    }

    // close process
    if (sambaQuery != NULL) {
        if (pclose(sambaQuery) != 0) {
            QString errorMsg;
            errorMsg.append("can't close process='read group list' ");
            errorMsg.append(cmd);
            perror(errorMsg);
        }
    }
}


/*
 * read available host from group with samba
 */
void makePopup::readHostList(QString group)
{
    hostBox -> clear();
//    hostBox -> insertItem(i18n("NO HOST"), -1);
    if (group == "NO GROUP") {
        return;
    }

    QString host;
    QString cmd;

    // search master of group
    cmd.append(SMBPATH);
    cmd.append("/smbclient -L ");
    cmd.append("localhost");
    cmd.append(" -N");

    // open process
    FILE *sambaQuery = popen(cmd.latin1(), "r");

    // read data
    if (sambaQuery != NULL) {
        QTextStream stream(sambaQuery, IO_ReadOnly);
        QString line;
        int groupLine = 0;
        int sepLine = 0;
        while (! stream.eof()) {
            line = stream.readLine();
            if (groupLine == 0 && line.contains("\tWorkgroup            Master", true) == 1) {
                groupLine = 1;
            } else if (groupLine == 1 && sepLine != 1 && line.contains("\t---------            -------", true) == 1) {
                sepLine = 1;
            } else if (sepLine == 1) {
                if (line.length() == 0) {
                    break;
                }
                line.remove(0, 1);
                QString tmp = line.mid(0, line.find("  ", 0, true));
                kdDebug() << "found group = '" << tmp << "'" << endl;
                if (tmp == group) {
                    line.remove(0, line.find("  ", 0, true));
                    host = line.stripWhiteSpace();
                    kdDebug() << "found master host= '" << host << "' in group = '" << tmp << "'" << endl;
                    break;
                }
            }
        }
    }
    else {
        QString errorMsg;
        errorMsg.append("can't open process='read master of group' ");
        errorMsg.append(cmd);
        perror(errorMsg);
    }

    // close process
    if (sambaQuery != NULL) {
        if (pclose(sambaQuery) != 0) {
            QString errorMsg;
            errorMsg.append("can't close process='read master of group' ");
            errorMsg.append(cmd);
            perror(errorMsg);
        }
    }

    // no master fround for group
    if (host.isEmpty()) {
        return;
    }

    // search hosts is group
    cmd.truncate(0);
    cmd.append(SMBPATH);
    cmd.append("/smbclient -L '");
    cmd.append(host);
    cmd.append("' -N");

    // open process
    sambaQuery = popen(cmd.latin1(), "r");

    // read data
    if (sambaQuery != NULL) {
        QTextStream stream(sambaQuery, IO_ReadOnly);
        QString line;
        int groupLine = 0;
        int sepLine = 0;
        while (! stream.eof()) {
            line = stream.readLine();
            if (groupLine == 0 && line.contains("\tServer               Comment", true) == 1) {
                groupLine = 1;
            } else if (groupLine == 1 && sepLine != 1 && line.contains("\t---------            -------", true) == 1) {
                sepLine = 1;
            } else if (sepLine == 1) {
                if (line.length() == 0) {
                    break;
                }
                line.remove(0, 1);
                host = line.mid(0, line.find("  ", 0, true));
                if (! host.isEmpty()) {
                    kdDebug() << "found host= '" << host << "' in group = '" << group << "'" << endl;
                    hostBox -> insertItem(host, -1);
                }
            }
        }
    }
    else {
        QString errorMsg;
        errorMsg.append("can't open process='read hosts from master' ");
        errorMsg.append(cmd);
        perror(errorMsg);
    }

    // close process
    if (sambaQuery != NULL) {
        if (pclose(sambaQuery) != 0) {
            QString errorMsg;
            errorMsg.append("can't close process='read hosts from master' ");
            errorMsg.append(cmd);
            perror(errorMsg);
        }
    }
}


/*
 * will be called, if groupbox is changed
 */
void makePopup::changeGroupBox(const QString &group)
{
    readHostList(group);
}
